###########################################################################
#
# This application plots data as recieved over a serial port
#
###########################################################################


import serial
import math
import wx
import time
import MyLog
import os
import sys
import _winreg as winreg
import itertools
import random

# These are order dependant. :(
import matplotlib
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigCanvas
from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg as NavigationToolbar
#import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import numpy as np
import pylab





###########################################################################
#
# Class to define the main application graph frame
#
###########################################################################
class GraphFrame(wx.Frame):
    """ The main frame of the application
    """
    title = 'Magnetometer'
    
    ###########################################################################
    # Constructor
    ###########################################################################
    def __init__(self, portName, refreshIntervalMs=10):

        wx.Frame.__init__(self, None, -1, self.title)

        self.port = None

        self.log = MyLog.MyLog("Mag")
        self.log.setFile()

        self.numParams        = 8
        self.refrehIntervalMs = refreshIntervalMs

        try:
          self.port = self.OpenSerial(portName)
        except:
          self.log.error("Unable to open serial port.")

          # Get list of ports
          ports = self.getComPortList()

          # Ask for port
          dlg = wx.SingleChoiceDialog(
                self, 'Dude, Pick a COM port!', 'Com Port Selection',
                ports, 
                wx.CHOICEDLG_STYLE
                )

          if dlg.ShowModal() == wx.ID_OK:

            portName = dlg.GetStringSelection()

            # Attempt to open this port
            try:
              self.port = self.OpenSerial(portName)
            except:
              self.port = None
              self.log.error("Unable to open serial port.")
            #

          else:
            self.port = None
          #

          dlg.Destroy()

        #

        if (self.port == None):
          print "Unable to open port"
          sys.exit(1)
        #

        self.log.debug("Opening port %s" % portName)

        # Create a list of lists each with one element in it (0)
        self.array = [0 for x in range(self.numParams)]
        for i in range(self.numParams):
          self.array[i] = np.zeros(1)
        #

        self.test = False
        self.Ax = 0.0
        self.Ay = 0.0
        self.Az = 0.0
        self.Mx = 0.0
        self.My = 0.0
        self.Mz = 0.0
        self.hdg = 0.0
        self.time = 0.0
        
        self.buf    = ""

        # Get a chunk of data from the source
        self.next()

        self.plotTypeIndex = 0
        self.dataTypeIndex = 1
        self.determineAxis()

        self.plotTypes = []
        self.plotTypes.append("XY")
        self.plotTypes.append("XZ")
        self.plotTypes.append("YX")
        self.plotTypes.append("YZ")
        self.plotTypes.append("ZX")
        self.plotTypes.append("ZY")
        self.plotTypes.append("3D")

        self.dataTypes = []
        self.dataTypes.append("Accel")
        self.dataTypes.append("Mag")

        self.create_menu()
        self.create_main_panel()
        
        self.redraw_timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.on_redraw_timer, self.redraw_timer)        
        self.redraw_timer.Start(self.refrehIntervalMs)

        self.plotType.SetItems(self.plotTypes)
        self.dataType.SetItems(self.dataTypes)

        self.plotType.SetSelection(self.plotTypeIndex)
        self.dataType.SetSelection(self.dataTypeIndex)

        self.Bind(wx.EVT_CLOSE, self.on_exit)

    #    

    ###################################################################
    #
    ###################################################################
    def determineAxis(self):
        if (self.dataTypeIndex == 0):   # ACCEL
            if self.plotTypeIndex == 0: # XY
                self.axisX = 1
                self.axisY = 2
                self.projection=None
            elif self.plotTypeIndex == 1: # XZ
                self.axisX = 1
                self.axisY = 3
                self.projection=None
            elif self.plotTypeIndex == 2: # YX
                self.axisX = 2
                self.axisY = 1
                self.projection=None
            elif self.plotTypeIndex == 3: # YZ
                self.axisX = 2
                self.axisY = 3
                self.projection=None
            elif self.plotTypeIndex == 4: # ZX
                self.axisX = 3
                self.axisY = 1
                self.projection=None
            elif self.plotTypeIndex == 5: # ZY
                self.axisX = 3
                self.axisY = 2
                self.projection=None
            elif self.plotTypeIndex == 6: # 3D
                self.axisX = 1
                self.axisY = 2
                self.axisZ = 3
                self.projection='3d'
            #
        else:
            if self.plotTypeIndex == 0: # XY
                self.axisX = 4
                self.axisY = 5
                self.projection=None
            elif self.plotTypeIndex == 1: # XZ
                self.axisX = 4
                self.axisY = 6
                self.projection=None
            elif self.plotTypeIndex == 2: # YX
                self.axisX = 5
                self.axisY = 4
                self.projection=None
            elif self.plotTypeIndex == 3: # YZ
                self.axisX = 5
                self.axisY = 6
                self.projection=None
            elif self.plotTypeIndex == 4: # ZX
                self.axisX = 6
                self.axisY = 4
                self.projection=None
            elif self.plotTypeIndex == 5: # ZY
                self.axisX = 6
                self.axisY = 5
                self.projection=None
            elif self.plotTypeIndex == 6: # 3D
                self.axisX = 4
                self.axisY = 5
                self.axisZ = 6
                self.projection='3d'
            #
        #
    #

        
    ###########################################################################
    # Get list on com ports
    ###########################################################################
    def getComPortList(self):
      comPorts = []

      path = 'HARDWARE\\DEVICEMAP\\SERIALCOMM'

      try:
        key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, path)
      except WindowsError:
        raise IterationError
      #

      for i in itertools.count():
        try:
          val = winreg.EnumValue(key, i)
          comPorts.append(str(val[1]))
        except:
          break
      #

      return comPorts
    #


        
    ###########################################################################
    # Open serial port
    ###########################################################################
    def OpenSerial(self, comName):
      sp = serial.Serial(comName)
      sp.baudrate = 115200
      sp.bytesize = 8
      sp.parity   = 'N'
      sp.stopbits = 1
      sp.timeout  = None
      sp.xonxoff  = 0
      sp.rtscts   = 0

  
      return sp
    #


    ###########################################################################
    # Close serial port
    ###########################################################################
    def CloseSerial(self, sp):
      sp.close()
    #


    ###########################################################################
    # Create the menu items
    ###########################################################################
    def create_menu(self):
        self.menubar = wx.MenuBar()
        
        menu_file = wx.Menu()
        m_expt = menu_file.Append(-1, "&Save plot\tCtrl-S", "Save plot to file")
        self.Bind(wx.EVT_MENU, self.on_save_plot, m_expt)
        menu_file.AppendSeparator()
        m_exit = menu_file.Append(-1, "E&xit\tCtrl-X", "Exit")
        self.Bind(wx.EVT_MENU, self.on_exit, m_exit)
                
        self.menubar.Append(menu_file, "&File")
        self.SetMenuBar(self.menubar)


    ###########################################################################
    # Create the main window
    ###########################################################################
    def create_main_panel(self):
        self.panel = wx.Panel(self)

        # Initialize the plot
        self.init_plot()
        self.canvas = FigCanvas(self.panel, -1, self.fig)

        # Create ComboBox box for plot type
        self.plotTypeText = wx.StaticText(self.panel, -1, "Plot Type:")
        self.plotType = wx.ComboBox(self.panel, 
                               500, 
                               "0", 
                               (90, 50), 
                               (160, -1), 
                               self.plotTypes,
                               wx.CB_DROPDOWN
                               )
        self.plotType.Bind(wx.EVT_COMBOBOX, self.on_plotType, self.plotType)

        # Create ComboBoxes boxes for parameter indices
        self.dataTypeText = wx.StaticText(self.panel, -1, "Data Type:")
        self.dataType = wx.ComboBox(self.panel, 
                               500, 
                               "1", 
                               (90, 50), 
                               (160, -1), 
                               self.dataTypes,
                               wx.CB_DROPDOWN
                               )
        self.dataType.Bind(wx.EVT_COMBOBOX, self.on_dataType, self.dataType)


        # Create the generic command entry box, text, and button
        self.Command = wx.TextCtrl(self.panel, -1, style=wx.TE_PROCESS_ENTER, size=(500, -1))
        self.Bind(wx.EVT_TEXT_ENTER, self.OnCommand, self.Command)        
        self.Command.SetValue("")
        self.CommandText = wx.StaticText(self.panel, -1, "Command:")


        # Place everything on the screen
        self.gbs = wx.GridBagSizer(1, 4) 

        self.reset = wx.Button(self.panel, -1, "Reset")
        self.reset.Bind(wx.EVT_BUTTON, self.on_reset)

        self.HeadingText = wx.StaticText(self.panel, -1, "123.3")

        #                                                        (Row,col)
        self.gbs.Add( self.dataType,                          (0,0) )
        self.gbs.Add( self.plotType,                          (0,1) )
        self.gbs.Add( self.reset,                             (0,2) )
        self.gbs.Add( self.HeadingText,                       (0,4) )

        # Create the horizontal command line
        self.hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        self.hbox1.AddSpacer(10)
        self.hbox1.Add(self.CommandText, border=5, flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL)
        self.hbox1.AddSpacer(10)
        self.hbox1.Add(self.Command,     border=5, flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
        self.hbox1.AddSpacer(10)
        self.hbox1.Fit(self)
          
        # Add the rows to the screen
        self.vbox = wx.BoxSizer(wx.VERTICAL)
        self.vbox.Add(self.canvas, 0, flag=wx.LEFT | wx.TOP | wx.GROW)        
        self.vbox.Add(self.gbs,    0, flag=wx.ALIGN_LEFT | wx.TOP)
        self.vbox.Add(self.hbox1,  0, flag=wx.ALIGN_LEFT | wx.TOP)
          
        self.panel.SetSizer(self.vbox)
        self.vbox.Fit(self)


    #

    
    ###########################################################################
    # Reset button callback
    ###########################################################################
    def OnCommand(self, event):

      cmd = self.Command.GetValue()
      self.log.debug("Sending user command: %s" % cmd)

      q = str('%s\r' % cmd)
      self.port.write(str(q))
    #


    ###########################################################################
    # Initialize the plots
    ###########################################################################
    def init_plot(self):

        # Set dots per inch
        self.dpi = 100

        # Create the figure 8"x3" at 100 dpi
        self.fig = Figure((8.0, 4.0), dpi=self.dpi)

        # Add a plot (row, columns, plot)
        self.axes = self.fig.add_subplot(1,2,1, projection=self.projection)
        self.axes.set_facecolor('black')
        
        pylab.setp(self.axes.get_xticklabels(), fontsize=8)
        pylab.setp(self.axes.get_yticklabels(), fontsize=8)

        self.axes3d = self.fig.add_subplot(1,2,2, projection='3d')
        self.axes3d.set_facecolor('grey')

        # Plot the data as a line series, and save the reference 
        # to the plotted line series
        # Plot returns a list of lines, but we only have one
        l = self.axes.plot(
            self.array[self.axisX], 
            self.array[self.axisY], 
            linewidth=1,
            color=(1, 1, 0),
            )
        self.plot_data = l[0]

        # Plot the data as a line series, and save the reference 
        # to the plotted line series
        # Plot returns a list of lines, but we only have one
        l = self.axes3d.plot(
            self.array[4], 
            self.array[5], 
            self.array[6], 
            linewidth=1,
            color=(1, 1, 0),
            )
        self.plot3d_data = l[0]

    #


    ###########################################################################
    # Redraw the plot
    ###########################################################################
    def draw_plot(self):

        # Get limits in all axis
        xmin = round(np.amin(self.array[self.axisX][0:]), 0)
        xmax = round(np.amax(self.array[self.axisX][0:]), 0)
        ymin = round(np.amin(self.array[self.axisY][0:]), 0)
        ymax = round(np.amax(self.array[self.axisY][0:]), 0)

        xmin = math.fabs(xmin)
        xmax = math.fabs(xmax)
        ymin = math.fabs(ymin)
        ymax = math.fabs(ymax)
        extent = max(xmin,xmax,ymin,ymax)
        if (extent < 0.1): extent = 1.0

        # Set the range of each axis
        self.axes.set_xbound(lower=-extent, upper=extent)
        self.axes.set_ybound(lower=-extent, upper=extent)
        
        # Set the axes grid
        self.axes.grid(True, color='gray')

        # Using setp here is convenient, because get_xticklabels
        # returns a list over which one needs to explicitly 
        # iterate, and setp already handles this.
        pylab.setp(self.axes.get_xticklabels(), visible=True)

        # Set the data 
        self.plot_data.set_xdata(self.array[self.axisX])
        self.plot_data.set_ydata(self.array[self.axisY])


        self.axes3d.cla()

        # Set the range of each axis
        self.axes3d.set_xlim3d(-extent, extent)
        self.axes3d.set_ylim3d(-extent, extent)
        self.axes3d.set_zlim3d(-extent, extent)

        self.axes3d.scatter(self.array[4], self.array[5], self.array[6],
            linewidth=1,
            color=(1, 1, 0))

        # Draw the figure
        self.canvas.draw()

        # Update the Heading text box
        self.HeadingText.SetLabel("%.1f"%self.array[7][-1])

    #
    

    ###########################################################################
    # When the user selects something, we go here.
    ###########################################################################
    def on_reset(self, evt):

        self.array = [0 for x in range(self.numParams)]
        for i in range(self.numParams):
          self.array[i] = np.zeros(1)
        #

    ###########################################################################
    # When the user selects something, we go here.
    ###########################################################################
    def on_plotType(self, evt):
        save = self.plotTypeIndex
        try:
            self.plotTypeIndex = evt.GetSelection()
        except:
            self.plotTypeIndex = save
        #
        self.determineAxis()
        self.plotType.SetItems(self.plotTypes)
        self.plotType.SetSelection(self.plotTypeIndex)
        self.draw_plot()
    #
    

    ###########################################################################
    # When the user selects something, we go here.
    ###########################################################################
    def on_dataType(self, evt):
        save = self.dataTypeIndex
        try:
            self.dataTypeIndex = evt.GetSelection()
        except:
            self.dataTypeIndex = save
        #
        self.determineAxis()
        self.dataType.SetItems(self.dataTypes)
        self.dataType.SetSelection(self.dataTypeIndex)
        self.draw_plot()
    #
    

    
    ###########################################################################
    # When a plot save is requested
    ###########################################################################
    def on_save_plot(self, event):

        file_choices = "PNG (*.png)|*.png"

        for i in range(999):
          name = "plot%04d.png" % i
          if (not os.path.isfile(name)):
            break;
        #
        
        dlg = wx.FileDialog(
            self, 
            message="Save plot as...",
            defaultDir=os.getcwd(),
            defaultFile=name,
            wildcard=file_choices,
            style=wx.SAVE)

        
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.canvas.print_figure(path, dpi=self.dpi)
        #
    #

    
    ###########################################################################
    # Updates on a timer to grab new data and plot it
    ###########################################################################
    def on_redraw_timer(self, event):
        
        # Try to get data from the source
        while (1):
          if (self.next() == 0):
            break
          #
        #

        # Redraw the plot
        self.draw_plot()
    #
    

    ###########################################################################
    # On a request to exit program
    ###########################################################################
    def on_exit(self, event):
        self.redraw_timer.Stop()
        time.sleep(0.2)
        self.CloseSerial(self.port)
        self.log.close()
        self.Destroy()
    #

    
    ###########################################################################
    # Determine if string is a valid integer
    ###########################################################################
    def is_integer(self, s):
        try:
            int(s)
            return True
        except ValueError:
            return False
        #
    #


    ############################################################################
    # Called to get the next array of numbers to plot
    # Returns 1 if it found a line, 0 otherwise
    ############################################################################
    def next(self):

      if (self.test):  
          self.time = self.time + 0.1
          self.Ax = 1.0 * math.sin(self.time)
          self.Ay = 1.0 * math.cos(self.time)
          self.Az = 1.0 * math.sin(self.time)
          self.Mx = 1.0 * math.sin(self.time)
          self.My = 1.0 * math.cos(self.time)
          self.Mz = 1.0 * math.sin(self.time)
          while (True):
              try:
                  self.Mx = random.uniform(-1,+1)
                  self.My = random.uniform(-1,+1)
                  self.Mz = math.sqrt(1.0*1.0 - self.Mx*self.Mx - self.My*self.My)
                  break
              except:
                  pass
          #
          self.Mz = self.Mz * random.choice( (-1.0, +1.0) )
          self.hdg = 1.0 * math.sin(self.time)
          cmd = "RUN,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f\n" % \
                  (self.Ax, self.Ay, self.Az, \
                   self.Mx, self.My, self.Mz, \
                   self.hdg)

          time.sleep(0.2)

      else:

        # Get next chunk of bytes
        n = self.port.inWaiting()
        if (n > 0):
          s = self.port.read(n)
          self.buf = self.buf + s
        #
  
        # Check the buffer for a terminator
        if ('\n' in self.buf):
          i = self.buf.find('\n')
        elif ('\r' in self.buf):
          i = self.buf.find('\r')
        else:
          return 0
        #
  
        cmd      = self.buf[0   : i-1]
        self.buf = self.buf[i+1 : ]
      #  


      # Split the buffer into tokens
      toks = cmd.split(',')

      # If this is RUN or CAL record
      if ("RUN" in toks[0]) or ("CAL" in toks[0]):

        # Determine size of data array
        l = len(self.array[0])

        # If the arrays are getting long
        if (l > 5000):

          # Chop them down
          for i in range(self.numParams):
            self.array[i] = self.array[i][4000:]
          #
          l = len(self.array[0])
        #


        # Pull data from tokens
        val = []
        try:
          tov = time.time()
          val.append(float(tov))
          val.append(float(toks[1])) # Ax
          val.append(float(toks[2])) # Ay
          val.append(float(toks[3])) # Az
          val.append(float(toks[4])) # Mx
          val.append(float(toks[5])) # My
          val.append(float(toks[6])) # Mz
          val.append(float(toks[7])) # Hdg
        except:
          return 0
        #

        for i in range(self.numParams):
            self.array[i] = np.append(self.array[i], val[i])
        #

      # If AAA

      cmd = cmd.strip()
      self.log.debug(cmd)

      if (self.test):
          return 0
      else:
          return 1

    #

# class GraphFrame



if __name__ == '__main__':
    app = wx.App(False)
    if (len(sys.argv) == 1):
      com = "COM2"
    elif (len(sys.argv) == 2):
      com = sys.argv[1]
    else:
      print "Invalid number of arguments"
      sys.exit()
    #


    app.frame = GraphFrame(com)
    app.frame.Show()
    app.MainLoop()

