/*****************************************************************************/
/*! @file Vectors.c
 * @brief Vector routines
 *
 *
 * @addtogroup MyLib
 * @{
 */
/*****************************************************************************/

/*****************************************************************************
 *                             Header files
 *****************************************************************************/
#include <math.h>
#include "vectors.h"

/*****************************************************************************
 *                             Constant defines
 *****************************************************************************/

/*****************************************************************************
 *                             Macro defines
 *****************************************************************************/

/*****************************************************************************
 *                             Enumerations
 *****************************************************************************/

/*****************************************************************************
 *                             Structures/unions
 *****************************************************************************/

/*****************************************************************************
 *                             File scoped function prototypes (static)
 *****************************************************************************/

/*****************************************************************************
 *                             Global scope variables
 *****************************************************************************/

/*****************************************************************************
 *                             Constant file statics (const)
 *****************************************************************************/

/*****************************************************************************
 *                             File scoped variables (static)
 *****************************************************************************/

/*****************************************************************************
 *                             Command parser definitions
 *****************************************************************************/

/*****************************************************************************
 *                             Function Definitions
 *****************************************************************************/



/*******************************************************************************
 ******************************************************************************/
/*! @brief Computes the cross product of two vectors
 *
 * Detailed Description
 *
 * @param[in]     a       Input vector
 * @param[in]     b       Input vector
 * @param[out]    out     Cross product of vectors
 * @return Nothing.
 ******************************************************************************/
void vector_cross(const vector_t *a,
                  const vector_t *b,
                  vector_t       *out)
{
  out->x = a->y*b->z - a->z*b->y;
  out->y = a->z*b->x - a->x*b->z;
  out->z = a->x*b->y - a->y*b->x;
}

/*******************************************************************************
 ******************************************************************************/
/*! @brief Computes the dot product of two vectors
 *
 * Detailed Description
 *
 * @param[in]     a       Index into thing
 * @param[out]    b       Number of things
 * @return The dot product
 ******************************************************************************/
float vector_dot(const vector_t *a,
                 const vector_t *b)
{
  return (a->x)*(b->x) + (a->y)*(b->y) + (a->z)*(b->z);
}

/*******************************************************************************
 ******************************************************************************/
/*! @brief Normalizes a vector to a unit vector
 *
 * Detailed Description
 *
 * @param[inout]     a       Vector to normalize
 * @return Nothing.
 ******************************************************************************/
vector_t  vector_normalize(vector_t *a)
{
  vector_t retval;
  float mag;

  mag = sqrt(a->x*a->x + a->y*a->y + a->z*a->z);
  if (mag > 0.0) {
    retval.x = a->x / mag;
    retval.y = a->y / mag;
    retval.z = a->z / mag;
  } else {
    Serial.println("Bad mag");
  }

  return retval;
}

/*! @} */
